#!/bin/env python

from pop import POP # http://starship.python.net/crew/pson/pop.html
import re, smtplib, glob, os, time, sys

import mimetypes
import email
from email import Encoders
from email.Message import Message
from email.MIMEAudio import MIMEAudio
from email.MIMEBase import MIMEBase
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email.MIMEText import MIMEText


DEBUG=True
def debug(message):
    if DEBUG:
        sys.stdout.write(message)

## the adress of the server is :
SENDER_ADDRESS='MyServer <myserver@free.fr>'

class MailerInterpreter:


    def __init__(self, mail_server,login,passwd):
	#debug("Connecting...")
	self.pop = POP(mail_server)
	self.pop.user(login,passwd)

        ## set defaults 
        self.share_path = "./"

        #debug("OK, ready. \n")

    def set_config(self, configDict):
        if type(configDict) is dict:
            self.config = configDict

    def set_share_path(self,aPath):
        self.share_path = aPath
        if aPath[len(aPath)-1] != "/":
            self.share_path += "/"
        
    def dispatch_subjects(self):
	#debug("Dispatching actions:\n")
	for id in range(1, len(self.pop)+1) :
            message_performed = False
	    subject_line = self.get_header("Subject:",id)
	    if re.match(r"^Subject:.*##help##.*",subject_line):
	      message_performed = self.send_help(id)
	    elif re.match(r"^Subject:.*##list##.*",subject_line):
	      message_performed = self.send_list(id)
	    elif re.match(r"^Subject:.*##get##.*",subject_line):
	      message_performed = self.send_file(id)
	    elif re.match(r"^Subject:.*##put##.*",subject_line):
	      message_performed = self.acquire_file(id)
	    ## SPECIAL
	    elif re.match(r"^Subject:.*##ftp##.*",subject_line):
	      message_performed = self.do_FTP(id)

            if not message_performed:
                debug("-%03d-> Message [%s] not performed\n" % (id,subject_line))
	    self.delete_mail(id)


	#debug("Done.\n")

    def get_header(self, header_name, mail_id):
	listOfHeaderLines = self.pop.top(mail_id)
	for line in listOfHeaderLines:
	    if line.find(header_name) != -1:
		return line
	return ""

    def split_header(self,header_line):
	separator = ": "
	separator_index = header_line.find(separator)
	if separator_index == -1:
	    return None
	return (header_line[:separator_index],
                header_line[separator_index+len(separator):])
    
    def extract_from_adress(self,mail_id):
	from_header = self.get_header("From:",mail_id)
	if from_header is None:
	    debug("\n[E]-No From in header\n")
	    return None
	(from_header,from_address) = self.split_header(from_header)
        return from_address
    
    #def __del__(self):
    #    self.quit()

    def quit(self):
	self.pop.quit()


    ## -- RESPONSE FOR ##help##
    def send_help(self,mail_id):
        help_message = """<html><body>
<h1>------ MAIL SERVER USAGE ------</h1>
if you send a mail with a keyword in the subject, the server acts and sends you a reply.
<table style="border-collapse: collapse; border: 1px solid gray;">
 <tr>
  <th>Keywork in subject</th><th>keyword in response</th><th>Actions</th>
 </tr>
 <tr>
  <td align="center"><pre>##help##</pre></td>
  <td align="center"><pre>##manual##</pre></td>
  <td>sends you this message</td>
 </tr>
 <tr>
  <td align="center"><pre>##list##</pre></td>
  <td align="center"><pre>##files## nb_files</pre></td>
  <td>sends you the list of available files</td>
 </tr>
 <tr>
  <td align="center"><pre>##get##&lt;file&gt;</pre></td> 
  <td align="center"><pre>##put##</pre></td>
  <td>sends you the file you requested ("put in mail")</td>
 </tr>
</table>
</body>
</html>
"""
        from_adress = self.extract_from_adress(mail_id)
        debug("%s-%03d-> Sending help to %s\n" % (time.strftime("%d/%m/%Y %H:%M"), mail_id,from_adress))
        self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       "##manual##", # subject
                       help_message,
                       attachedFile = None,
                       messageFormat='html') # mode
        return True

    ## -- RESPONSE FOR ##list##
    def send_list(self,mail_id):
        fileList = glob.glob(self.share_path + "*")
        list_message = ""
        for fileName in fileList:
            shortFilename = os.path.split(fileName)[1]
            list_message += "<%s> %s\n" % (shortFilename, os.path.getsize(fileName) )
        
        from_adress = self.extract_from_adress(mail_id)
        subject = "##files## %s" % len(fileList)
        debug("%s-%03d-> Sending list to %s\n" % (time.strftime("%d/%m/%Y %H:%M"),mail_id,from_adress))
        self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       subject, # subject
                       list_message)
        return True

    ## -- RESPONSE FOR ##get##
    def send_file(self,mail_id):
        from_adress = self.extract_from_adress(mail_id)
        fullSubject = self.get_header("Subject",mail_id)
	separator = "##"
	firstSeparatorIndex = fullSubject.find(separator)
	nextSeparatorIndex = fullSubject.find(separator,firstSeparatorIndex+len(separator))
	fullSubject = fullSubject[nextSeparatorIndex+len(separator):]
	fileFirstSeparator = fullSubject.find("<")
	fileLastSeparator = fullSubject.rfind(">")
	if (fileFirstSeparator == -1) or (fileLastSeparator == -1):
	    debug("\n[E]-Wrong usage of ##get##\n")
	    return False
        filename = fullSubject[fileFirstSeparator+1:fileLastSeparator]
	filename = filename.replace("\t"," ")
        if len(filename) == 0:
	    debug("[E]-Empty filename.\n")
	    return False
        
	startOctet = None
	stopOctet = None
	splitInfo = fullSubject[fileLastSeparator+1:].strip()
	try:
	    if len(splitInfo) > 0:
		stopOctetIndex = splitInfo.find("-")
		startOctet = splitInfo[:stopOctetIndex].strip()
		stopOctet = splitInfo[stopOctetIndex+1:].strip()
		sys.stdout.write("[%s]-[%s]" % (startOctet, stopOctet))
		if len(startOctet) == 0:
		    startOctet = None
		else:
		    startOctet = int(startOctet)
		    if len(stopOctet) == 0:
			stopOctet = None
		    else:
			stopOctet = int(stopOctet)
	except StandardError:
	    debug("\n[E]-Can not read split info...\n")
	    startOctet = None
	    stopOctet = None
        debug("%s-%03d-> Sending file <%s> to %s\n" % (time.strftime("%d/%m/%Y %H:%M"),mail_id,filename,from_adress))
	content = "File <%s>" % filename
	if (startOctet is not None) and (stopOctet is not None) :
	    content = "File <%s> %s-%s" % (filename,startOctet,stopOctet)
	if (startOctet is None) and (stopOctet is None):
	    subject = "##put## <%s>" % filename
	else:
	    subject = "##put## <%s> %s-%s" % (filename,startOctet,stopOctet)
	self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       subject, # subject
                       content,  # message body
                       filename,           # attached file
		       None,               # messageFormat
		       startOctet,         # start of file offset
		       stopOctet)          # end of file offset

        return True

    ## -- RESPONSE FOR ##put##
    def acquire_file(self,mail_id):
        from_adress = self.extract_from_adress(mail_id)
        debug("%s-%03d-> Acquiring file from %s\n" % (time.strftime("%d/%m/%Y %H:%M"),mail_id,from_adress))
        messageString = "\n".join(self.pop[mail_id])
        mainMessage = email.message_from_string(messageString)
        allFiles = []
        for attachedMessage in mainMessage.walk():
            if attachedMessage.is_multipart():
                continue
            filename = attachedMessage.get_filename()
            if filename is None:
                continue
            file = open(self.share_path + filename,'wb')
            file.write(attachedMessage.get_payload(decode=True))
            file.close()
            allFiles.append(filename)
        return True

    def do_FTP(self, mail_id):
        from_adress = self.extract_from_adress(mail_id)
        subject = "##ftp-log## "
        debug("%s-%03d-> FTP : from %s\n" % (time.strftime("%d/%m/%Y %H:%M"),mail_id,from_adress))

        messageString = "\n".join(self.pop[mail_id])
        mainMessage = email.message_from_string(messageString)
        for attachedMessage in mainMessage.walk():
	    str_message = "%s" % attachedMessage
	    ftp_content = str_message[str_message.find("\n\n")+2:]
	    if len(ftp_content) > 0:
		break
	ftp_file = open("/home/julien/devel/mail/ftp.sh","w")
	ftp_file.write("""#!/bin/sh
cd /home/guest/
rm -f ftp-log.txt
ftp -n 1>> ftp-log.txt 2>>ftp-log.txt << ____FTP_OEF____
""")
        ftp_file.write(ftp_content)
        ftp_file.write("\nbye\n____FTP_OEF____")
	ftp_file.close()
	os.system("/bin/sh /home/julien/devel/mail/ftp.sh")
	#&& rm /home/julien/devel/mail/ftp.sh")
	log_file = open("/home/guest/ftp-log.txt")
	message = log_file.read()
	log_file.close()
	self.send_mail(SENDER_ADDRESS,     # from
                       from_adress,        # to
                       subject, # subject
                       message)
        return True

    
    def delete_mail(self,mail_id):
        del self.pop[mail_id]
	debug("%s-%03d Message deleted.\n" % (time.strftime("%d/%m/%Y %H:%M"),mail_id))
        
        

    def send_mail(self,
                  from_address,
	          to_address,
		  subject,
		  message,
		  attachedFile = None,
		  messageFormat = 'text',
		  startOffset = None,
		  stopOffset = None):
        outer = ""
        if attachedFile is None:
	    if messageFormat == 'text':
		outer = MIMEText(message)
	    else:
		outer = MIMEBase('multipart','mixed')
		outer.attach(MIMEText(message,messageFormat))
        else:
            outer = MIMEMultipart()
            outer.attach(MIMEText(message))
            outer.preamble = 'You will not see this in a MIME-aware mail reader.\n'
            outer.epilogue = ''
	    self.__attach_file(self.share_path+attachedFile, # path to file 
	                       outer,                        # message to update
			       attachedFile,                 # name in mail
			       startOffset,
			       stopOffset
			       )

        outer['Subject'] = subject
        outer['To'] = to_address
        outer['From'] = from_address
        server = smtplib.SMTP('smtp.neuf.fr')
        server.sendmail(from_address,
                        [to_address],
                        outer.as_string()
                        )
        server.quit()

    ##----------------------------------------------
    ## PRIVATE METHODS
    def __attach_file(self, path, outer, filename, startOffset, stopOffset):
	#COMMASPACE = ', '
        # Guess the content type based on the file's extension.  Encoding
        # will be ignored, although we should check for simple things like
        # gzip'd or compressed files.
        ctype, encoding = mimetypes.guess_type(path)
        if ctype is None or encoding is not None:
            # No guess could be made, or the file is encoded (compressed), so
            # use a generic bag-of-bits type.
            ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        if maintype == 'text':
	    fileContent = self.__extract_file_content(path,'r',startOffset,stopOffset)
            # Note: we should handle calculating the charset
            msg = MIMEText(fileContent, _subtype=subtype)
        elif maintype == 'image':
	    fileContent = self.__extract_file_content(path,'rb',startOffset,stopOffset)
            msg = MIMEImage(fileContent, _subtype=subtype)
        elif maintype == 'audio':
	    fileContent = self.__extract_file_content(path,'rb',startOffset,stopOffset)
            msg = MIMEAudio(fileContent, _subtype=subtype)
        else:
	    fileContent = self.__extract_file_content(path,'rb',startOffset,stopOffset)
            msg = MIMEBase(maintype, subtype)
            msg.set_payload(fileContent)
            # Encode the payload using Base64
            Encoders.encode_base64(msg)
        # Set the filename parameter
        msg.add_header('Content-Disposition', 'attachment', filename=filename)
        outer.attach(msg)

    def __extract_file_content(self, pathToFilename, mode, startOffset, stopOffset):
	fileContent = ""
	try:
	    fp = open(pathToFilename, mode)
	    if startOffset is not None and stopOffset is not None:
		if startOffset < stopOffset:
		    fp.seek(startOffset)
		    fileContent = fp.read(stopOffset-startOffset)
	    elif startOffset is None and stopOffset is not None:
		fileContent = fp.read(stopOffset)
	    elif startOffset is not None and stopOffset is None:
		fp.seek(startOffset)
		fileContent = fp.read()
	    else:
		fileContent = fp.read()
	    fp.close()
	except StandardError, raison:
	    print raison
	return fileContent
	
MAIL_SERVER='pop.free.fr'
LOGIN='myserver.login'
PASSWD='myserver.passwd'
interpreter = MailerInterpreter(MAIL_SERVER,LOGIN,PASSWD)
interpreter.set_share_path("/home/share/")

interpreter.dispatch_subjects()
interpreter.quit()
